import json, shlex, subprocess, io, sys, os
import os.path
try:
    from pypm.post_operations import update_package_json_after_operation, update_package_json_after_uninstall
except ImportError:
    from .post_operations import update_package_json_after_uninstall, update_package_json_after_operation

cur_path = str(os.getcwd())

class PyPM:
    def __init__(self):
        self.path = cur_path
        self.verbose = True
        self.service = 'pip'
        self.arguments = None
        self.package_json = None

    def set_variables(self, path, verbose, service, arguments):
        self.path = path
        self.verbose = verbose
        self.service = service
        self.arguments = arguments

    def __reader__(self):
        try:
            with open(os.path.join(self.path, 'package.json'), 'r', encoding='utf8') as packagejson:
                return json.loads(packagejson.read())
        except FileNotFoundError:
            print('No package.json found. Please run pypm init')
            exit()

    def __commander__(self, key, item):
        arguments = self.arguments if self.arguments != None else ''
        option = {
            'run': shlex.split(item),
            'install': shlex.split(f'{self.service} install {arguments} {item}'),
            'uninstall': shlex.split(f'{self.service} uninstall {arguments} {item}'),
            'update': shlex.split(f'{self.service} install --upgrade {item}') if 'pip' in self.service else shlex.split(f'{self.service} update {arguments} {item}')
        }.get(key)
        return option

    def __get_dependencies__(self):
        try:
            depends = list(self.package_json['dependencies'].keys())
            depends.extend(list(self.package_json['devDependencies'].keys()))
            return self.__list_to_str__(depends)
        except KeyError as ke:
            if ke == 'devDependencies':
                return self.__list_to_str__(list(self.package_json['dependencies'].keys()))
            else:
                print('No dependencies found.')
                sys.exit()

    def __process__(self, cmd):
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        for line in io.TextIOWrapper(proc.stdout):
            if self.verbose:
                print(line.replace('\n', ''))

    def __list_to_str__(self, item_list):
        return ' '.join(item_list)

    def __pretty_key__(self, key):
        return key.replace(' ', ', ')
    
    def __assign_package_json__(self):
        self.package_json = self.__reader__()

    def start(self):
        self.__assign_package_json__()
        if self.verbose:
            print('Running start command...\n')
        cmd = self.__commander__('run', str(self.package_json['scripts']['start']))
        self.__process__(cmd)
    
    def run(self, key):
        self.__assign_package_json__()
        if self.verbose:
            print(f'Running {key}...\n')
        cmd = self.__commander__('run', str(self.package_json['scripts'][str(key)]))
        self.__process__(cmd)
    
    def install(self, key):
        key = self.__list_to_str__(key)
        self.__assign_package_json__()
        if len(key) == 0:
            if self.verbose:
                print('Installilng all dependencies...')
            cmd = self.__commander__('install', self.__get_dependencies__())
        else:
            if self.verbose:
                print(f'Installing package(s) {self.__pretty_key__(key)}...\n')
            cmd = self.__commander__('install', key)
        self.__process__(cmd)
        if len(key) != 0:
            update_package_json_after_operation(key, self.path, self.package_json)
    
    def uninstall(self, key):
        key = self.__list_to_str__(key)
        self.__assign_package_json__()
        if len(key) == 0:
            if self.verbose:
                print('Uninstalling all dependencies...\n')
            cmd = self.__commander__('uninstall', self.__get_dependencies__())
        else:
            if self.verbose:
                print(f'Uninstalling package(s) {self.__pretty_key__(key)}...\n')
            cmd = self.__commander__('uninstall', key)
        self.__process__(cmd)
        if len(key) != 0:
            update_package_json_after_uninstall(key, self.path, self.package_json)
    
    def update(self, key):
        key = self.__list_to_str__(key)
        self.__assign_package_json__()
        if len(key) == 0:
            if self.verbose:
                print('Updating all dependencies...\n')
            cmd = self.__commander__('update', self.__get_dependencies__())
        else:
            if self.verbose:
                print(f'Updating package(s) {self.__pretty_key__(key)}...\n')
            cmd = self.__commander__('update', key)
        self.__process__(cmd)
        if len(key) != 0:
            update_package_json_after_operation(key, self.path, self.package_json)



import argparse, sys, os, click
try:
    from .version import __version__
    from .main import PyPM
    from .generator import Generator
except ImportError:
    from pypm.version import __version__
    from pypm.main import PyPM
    from pypm.generator import Generator

pypm = PyPM()
generator = Generator()

def service_check(arg):
    available_option = {
        'pip': True,
        'pip3': True,
        'npm': True,
        'npx': True
    }.get(arg, False)
    if not available_option:
        print(f'[!] Error [!] Invalid service: {arg}')
        sys.exit(1)


@click.group()
@click.option('--path', type=str, default=str(os.getcwd()), help='Path of package.json. Defaults to current directory CLI tool is called from.')
@click.option('--verbose', type=bool, default=True, help='Message output')
@click.option('--service', type=str, default='pip', help='Which service to use (pip, pip3, npm, npx)')
@click.option('--arguments', type=str, help='Extra pip, npm or npx arguments to append to commands.')
@click.version_option(version=__version__)
def cli(path, verbose, service, arguments):
    """Python package manager for projects running Python3.6 and above."""
    service_check(service)
    pypm.set_variables(path, verbose, service, arguments)


@cli.command()
@click.argument('path', nargs=1, default=os.getcwd())
@click.argument('verbose', nargs=-1)
def init(path, verbose):
    try:
        if len(verbose) == 0:
            verbose = True
        generator.set_variables(path, verbose)
        generator.generate()
    except Exception as e:
        click.echo(f'Failed to generate package.json. Error: \n', e)


@cli.command()
@click.argument('script')
def run(script):
    pypm.run(script)

@cli.command()
def start():
    pypm.start()

@cli.command()
@click.argument('dependency', nargs=-1)
def install(dependency):
    pypm.install(dependency)

@cli.command()
@click.argument('dependency', nargs=-1)
def uninstall(dependency):
    pypm.uninstall(dependency)

@cli.command()
@click.argument('dependency', nargs=-1)
def update(dependency):
    pypm.update(dependency)


if __name__ == '__main__':
    cli()



import json, re, os, sys, subprocess, io, shlex, time
import os.path, pkg_resources
from stdlib_list import stdlib_list

class Generator:
    """
        This class generates a package.json for a given python project.
        The package.json is structured similar to npm's package.json. It will try
        to fetch information from requirements.txt and setup.py. If neither are
        present the generator will terminate.
    """
    def __init__(self):
        self.path = str(os.getcwd())
        self.verbose = True
        self.valid_version = True
        self.default_packages = stdlib_list(f'{sys.version[0:3]}')
        self.self_generated_reqs = False

    def set_variables(self, path, verbose):
        self.path = path
        self.verbose = verbose

    def __writer__(self, data):
        with open(os.path.join(self.path, 'package.json'), 'w', encoding='utf8') as packagejson:
            data = json.dumps(data, indent=4, ensure_ascii=True)
            packagejson.write(data)
    
    def __reader__(self, filename, datatype=None):
        try:
            with open(os.path.join(self.path, filename), 'r', encoding='utf8') as reader:
                if datatype is None:
                    return reader.readlines()
                elif datatype == 'json':
                    return json.loads(reader.read())
        except FileNotFoundError:
            # print(f'{filename} Not Found. Please ensure you have both setup.py and requirements.txt in your root directory.')
            return None

    def __organize_requirements__(self, req_file, count=2):
        obj = {}
        for env in req_file:
            en_v = re.sub("['\"]", '', env.replace('\n', ''))
            if en_v.find('>') == -1:
                idx = en_v.find('=')
            else:
                idx = en_v.find('>')
            obj[re.sub('\s+', '', str(en_v[:idx]))] = str(en_v[idx+count:])
        return obj
    
    def __get_user_input__(self, no_setup_py=False):
        obj = {
            'scripts': {}
        }
        print('Press press enter to leave as default.\n')
        if no_setup_py:
            questions = [
                f'Enter name of project (default: {os.path.basename(self.path)}) > ',
                'Enter version (default: 0.0.1) > ',
                'Enter description (default: blank) > ',
                'Enter author (default: blank) > ',
                'Enter entry point file (default: main.py) > ',
                'Enter start script (default: python [entry point file]) > ',
                'Enter test script (default: blank) > ',
                'Enter license (default: ISC) > '
            ]
            for index, question in enumerate(questions):
                answer = input(question)
                if index == 0:
                    obj['name'] = answer if answer != '' else f'{os.path.basename(self.path)}'
                elif index == 1:
                    obj['version'] = answer if answer != '' else '0.0.1'
                elif index == 2:
                    obj['description'] = answer if answer != '' else ''
                elif index == 3:
                    obj['author'] = answer if answer != '' else ''
                elif index == 4:
                    if answer == '':
                        obj['main'] = 'main.py'
                    obj['main'] = answer if answer != '' else f'{obj['main']}'
                elif index == 5:
                    obj['scripts']['start'] = answer if answer != '' else f"python {obj['main']}"
                elif index == 6:
                    obj['scripts']['test'] = answer if answer != '' else 'echo "No test available"'
                elif index == 7:
                    obj['license'] = answer if answer != '' else 'ISC'
        else:
            questions = [
                'Enter version (default: 0.0.1) > ',
                'Enter entry point file (default: main.py) > ',
                'Enter start script (default: python [entry point file]) > ',
                'Enter test script (default: blank) > ',
                'Enter license (default: ISC) > '
            ]
            for index, question in enumerate(questions):
                answer = input(question)
                if index == 0:
                    obj['version'] = answer if answer != '' else '0.0.1'
                elif index == 1:
                    if answer == '':
                        obj['main'] = 'main.py'
                    obj['main'] = answer if answer != '' else f'{obj['main']}'
                elif index == 2:
                    obj['scripts']['start'] = f"python {obj['main']}" if answer != '' else "python main.py"
                elif index == 3:
                    obj['scripts']['test'] = answer if answer != '' else 'echo "No test available"'
                elif index == 4:
                    obj['license'] = answer if answer != '' else 'ISC'
        return obj
        
    
    def __check_file_system__ (self, filename):
        if os.path.isfile(os.path.join(self.path, 'package.json')):
            print('package.json already exists for this project.')
            exit()
        else:
            if not os.path.isfile(os.path.join(self.path, filename)):
                command = shlex.split(f'pipreqs {self.path}')
                try:
                    proc = subprocess.Popen(command, stdout=subprocess.PIPE, env=os.environ.copy(), stderr=subprocess.PIPE)
                    io.TextIOWrapper(proc.stderr, encoding='utf8', newline='')  # stops error messages from pipreqs from displaying
                    self.self_generated_reqs = True
                    time.sleep(4) # allow time to generate requirements.txt
                except Exception:
                    print('Unable to generate requirements.txt. Please generate manually and proceed as normal.')
                    exit()

    def _create_structure_(self, setup_py, template_package_json, req_dependencies):
        template_package_json['name'] = setup_py['name']
        template_package_json['version'] = setup_py['version']
        if '.' not in setup_py['version']:
            self.valid_version = False
        template_package_json['description'] = setup_py['description']
        template_package_json['dependencies'].update(req_dependencies)
        template_package_json['author'] = setup_py['author']
        return template_package_json
    
    def generate(self):
        if self.verbose:
            print('Running automated data retrieval tool. One moment...')
        self.__check_file_system__('requirements.txt')
        req_dependencies = self.__organize_requirements__(self.__reader__('requirements.txt'))
        setup_py_contents = self.__reader__('setup.py')
        resource_package = __name__
        resource_path = '/'.join(('data', 'pkg.json'))
        template = pkg_resources.resource_string(resource_package, resource_path).decode('utf-8')
        template_package_json = json.loads(template)
        valid_meta = []
        noSetupPy = False
        if setup_py_contents is not None:
            for line in setup_py_contents:
                for keyword in ['name', 'author', 'description', 'url', 'version']:
                    if re.search(keyword + '=', line):
                        valid_meta.append(re.sub("['\"]", '', line.replace(',', '').replace('\n', '')))
            setup_py = self.__organize_requirements__(valid_meta, 1)
            template_package_json = self._create_structure_(setup_py, template_package_json, req_dependencies)
        else:
            noSetupPy = True
        if self.verbose:
            print('Automated data retrieval complete. Please follow on screen instructions below.\n')
        template_package_json.update(self.__get_user_input__(no_setup_py=noSetupPy))
        if self.verbose:
            print('Writing package.json...')
            print('Write to file complete.')
        self.__writer__(template_package_json)
        if self.self_generated_reqs:
            os.remove(os.path.join(self.path, 'requirements.txt'))



import setuptools
import os.path as path
from pypm.version import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt', 'r') as reqs:
    dependencies = reqs.readlines()

setuptools.setup(
    name="pypm2",
    version=__version__,
    author="AbleInc",
    author_email="douglas.jaylen@gmail.com",
    description="Python package manager for projects running Python3.6 and above.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ableinc/pypm",
    keywords=['package manager', 'dependency manager', 'manager', 'python 3', 'cli tool', 'command line tool'],
    packages=setuptools.find_packages(),
    package_data={
      'pypm': ['pypm/data/pkg.json']
    },
    data_files=[
        ('/pypm/data', [path.join('pypm/data', 'pkg.json')])
    ],
    entry_points='''
        [console_scripts]
        pypm=pypm.cli:cli
    ''',
    install_requires=dependencies,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)